package be.seeseemelk.llamascript;

import be.seeseemelk.llamascript.arguments.ArgumentBag;
import be.seeseemelk.llamascript.arguments.ListArgumentBag;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@RequiredArgsConstructor
public class LlamaScript {
    private final LlamaScriptWorker worker;

    public <T> T eval(String prompt, Class<T> returnType, ArgumentBag arguments) {

        if (returnType.equals(Integer.class)) {
            //noinspection unchecked
            return (T) evalInt(prompt, arguments);
        } else if (returnType.equals(String.class)) {
            //noinspection unchecked
            return (T) evalString(prompt, arguments);
        } else if (returnType.equals(Boolean.class)) {
            //noinspection unchecked
            return (T) evalBool(prompt, arguments);
        } else {
            throw new LLamaScriptException("Unsupported return type");
        }
    }

    public String evalString(String prompt, ArgumentBag arguments) {
        var result = worker.eval(prompt, primitiveOutputType("a string"), arguments);
        if (result instanceof String string)
            return string;
        else if (result != null)
            return result.toString();
        else
            throw new LLamaScriptException("No response");
    }

    public String evalString(String prompt, Object... arguments) {
        return eval(prompt, String.class, ListArgumentBag.of(arguments));
    }

    public Integer evalInt(String prompt, ArgumentBag arguments) {
        var result = worker.eval(prompt, primitiveOutputType("an integer number"), arguments);
        if (result instanceof Integer integer)
            return integer;
        else if (result instanceof String string) {
            try {
                return Integer.parseInt(string);
            } catch (NumberFormatException e) {
                throw new LLamaScriptException("Invalid integer", e);
            }
        }
        else
            throw new LLamaScriptException("Invalid integer type");
    }

    public int evalInt(String prompt, Object... arguments) {
        return eval(prompt, Integer.class, ListArgumentBag.of(arguments));
    }

    public Boolean evalBool(String prompt, ArgumentBag arguments) {
        var result = worker.eval(prompt, primitiveOutputType("a boolean value"), arguments);
        if (result instanceof Boolean bool)
            return bool;
        else if (result instanceof String string)
            return Boolean.parseBoolean(string);
        else
            throw new LLamaScriptException("Invalid boolean type");
    }

    public boolean evalBool(String prompt, Object... arguments) {
        return eval(prompt, Boolean.class, ListArgumentBag.of(arguments));
    }

    private static String primitiveOutputType(String description) {
        return "This result must contain %s.".formatted(description);
    }
}
