package be.seeseemelk.llamascript;

import be.seeseemelk.llamascript.arguments.ArgumentBag;
import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import com.fasterxml.jackson.databind.ObjectMapper;
import io.github.ollama4j.OllamaAPI;
import io.github.ollama4j.exceptions.OllamaBaseException;
import io.github.ollama4j.models.chat.OllamaChatMessage;
import io.github.ollama4j.models.chat.OllamaChatMessageRole;
import io.github.ollama4j.models.chat.OllamaChatRequest;
import io.github.ollama4j.utils.OptionsBuilder;
import lombok.Builder;

import java.io.IOException;
import java.util.ArrayList;

@Builder
public class LlamaScriptWorker {
    private final OllamaAPI api;
    private final String model;
    private final ObjectMapper mapper = new ObjectMapper();

    public Object eval(String prompt, String outputType, ArgumentBag arguments) {
        try {
            var optionsBuilder = new OptionsBuilder();
            optionsBuilder.setSeed(20); // Chosen with 1d20. Yes, a nat 20! Let's go baby!
            optionsBuilder.setTemperature(0);
            var options = optionsBuilder.build();
            var fullPrompt = new StringBuilder("""
                You will receive a prompt, and will have to consider the question in the prompt with regards to the inputs you will receive.
                Your output must be a JSON object containing the single value 'result'.
                Your answer must be correct. Do not make mistakes
                %s
                
                The prompt: %s
                
                Arguments:
                """.formatted(outputType, prompt));
            arguments.writeTo(fullPrompt);
            System.out.printf("Prompt is: %s\n", fullPrompt);

            var messages = new ArrayList<OllamaChatMessage>();
            messages.add(new OllamaChatMessage(OllamaChatMessageRole.USER, fullPrompt.toString()));
            var chatRequest = new OllamaChatRequest(model, messages);
            chatRequest.setReturnFormatJson(true);
            chatRequest.setOptions(options.getOptionsMap());

            var result = api.chat(chatRequest);
            System.out.printf("Result is: %s\n", result.getResponse());
            return mapper.readValue(result.getResponse(), Result.class).result;
        } catch (OllamaBaseException | InterruptedException | IOException e) {
            throw new LLamaScriptException(e);
        }
    }

    @JsonIgnoreProperties(ignoreUnknown = true)
    private static class Result {
        public Object result;
    }
}
