package be.seeseemelk.llamascript.arguments;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * A bag of arguments that will keep the order in which the arguments were added.
 * The arguments are added with an index, which starts at one for the first argument.
 */
public class ListArgumentBag implements ArgumentBag {
    private final List<Object> values = new ArrayList<>();

    /**
     * Adds an argument to the bag.
     * @param value The argument to add.
     * @return This instance.
     */
    public ListArgumentBag add(Object value) {
        values.add(value);
        return this;
    }

    @Override
    public void writeTo(StringBuilder builder) {
        for (int i = 0; i < values.size(); i++) {
            builder.append("[").append(i + 1).append("] -> ").append(Objects.toString(values.get(i))).append('\n');
        }
    }

    /**
     * Creates an argument bag from an array of values.
     * @param values The values to add.
     * @return The bag with the arguments added.
     */
    public static ListArgumentBag of(Object... values) {
        var bag = new ListArgumentBag();
        for (var value : values) {
            bag.add(value);
        }
        return bag;
    }
}
